package newCynic;
/**
 *      ""
 */
public abstract class DriverState {

    /**  ,    */
    private String _stateName;

    /** ,  .     . */
    protected DriverState(String aStateName) {
        _stateName = aStateName;
    }
    /** ,   . */
    public String getName() {
        return _stateName;
    }
    /** ,  .   
     *        .
     */
    public abstract void processEvent(int aEvent, Driver aDriver);

    /** ,       .  
     *         .
     */
    public abstract void onEnter(Driver aDriver);

    /**  0 --  "".*/
    private final static DriverState STATE_0_PENDULUM = new DriverState0();
    /**  1 --  "".*/
    private final static DriverState STATE_1_ARC = new DriverState1();
    /**  2 --  "".*/
    private final static DriverState STATE_2_DIGRESSION = new DriverState2();
    /**  3 --  "" ( ).*/
    private final static DriverState STATE_3_FINISH = new DriverState3();

    /**  ,   
     *  (     ).
     */
    public static void reset (Driver aDriver) {
        aDriver.setCurrentState(STATE_0_PENDULUM);
    }

    /**    --  . */
    private static void doStartLogging(int aEvent, Driver aDriver) {
        if (Constants.OBJECTS_LOGGING)
            Logger.log("  '':");
        if (Constants.A3_BEGIN_LOGGING)
            Logger.logBegin(Constants.DRIVER_AUTOMATE_NAME, aDriver.getCurrentState().getName(), aEvent);
    }

    /**    --  . */
    private static void doEndLogging(int aEvent, Driver aDriver) {
        if (Constants.A3_END_LOGGING)
            Logger.logEnd(Constants.DRIVER_AUTOMATE_NAME, aDriver.getCurrentState().getName());
    }
    /**   ,  .  */
    private static void changeState(DriverState aNewState, Driver aDriver) {
        if (Constants.A3_TRANS_LOGGING)
            Logger.logStateChange(Constants.DRIVER_AUTOMATE_NAME, aNewState.getName(), aDriver.getCurrentState().getName());
        aDriver.setCurrentState(aNewState);
        aNewState.onEnter(aDriver);

    }

    /**
      *  ,    aEvent  aDriver.
     *     .
      */
    public static void processIncomingEvent(int aEvent, Driver aDriver) {
        doStartLogging(aEvent, aDriver);
        aDriver.getCurrentState().processEvent(aEvent, aDriver);
        doEndLogging(aEvent, aDriver);
    }

    /** ,   0  "" */
    private static class DriverState0 extends DriverState {

        public DriverState0() {
            super("State 0");
        }

        public void processEvent(int aEvent, Driver aDriver) {
            if ((!aDriver.x100_enemyIsNear()) && (!aDriver.x105_wallIsNear())) {
                changeState(STATE_1_ARC, aDriver);
            } else if (aDriver.x110_timeoutExpired()) {
                aDriver.z200_0_initializePendulumTrajectory();
                aDriver.z200_1_randomizePendulumTrajectory();
                aDriver.z200_2_calculatePendulumTrajectory();
            } else {
                aDriver.z200_1_randomizePendulumTrajectory();
                aDriver.z200_2_calculatePendulumTrajectory();
            }
        }

        public void onEnter(Driver aDriver) {
            aDriver.z200_0_initializePendulumTrajectory();
            aDriver.z200_1_randomizePendulumTrajectory();
            aDriver.z200_2_calculatePendulumTrajectory();
        }
    }

    /** ,   1  "" */
    private static class DriverState1 extends DriverState {

        public DriverState1() {
            super("State 1");
        }

        public void processEvent(int aEvent, Driver aDriver) {
            if (aEvent == Constants.EVENT_HIT_BY_BULLET) {
                changeState(STATE_2_DIGRESSION, aDriver);
            } else if (aDriver.x100_enemyIsNear() || aDriver.x105_wallIsNear() || aEvent == Constants.EVENT_ENEMY_COLLISION || aEvent == Constants.EVENT_WALL_COLLISION) {
                changeState(STATE_0_PENDULUM, aDriver); //Bug found!!!!!!!!!!
            } else if (aDriver.x110_timeoutExpired()) {
                changeState(STATE_3_FINISH, aDriver);
            } else {
                aDriver.z210_1_randomizeArcTrajectory();
                aDriver.z210_2_calculateArcTrajectory();
            }
        }

        public void onEnter(Driver aDriver) {
            aDriver.z210_0_initializeArcTrajectory();
            aDriver.z210_1_randomizeArcTrajectory();
            aDriver.z210_2_calculateArcTrajectory();
        }
    }
    /** ,   2  "" */

    private static class DriverState2 extends DriverState {

        public DriverState2() {
            super("State 2");
        }

        public void processEvent(int aEvent, Driver aDriver) {
            if (aDriver.x105_wallIsNear() || aDriver.x110_timeoutExpired() || aEvent == Constants.EVENT_ENEMY_COLLISION || aEvent == Constants.EVENT_WALL_COLLISION) {
                changeState(STATE_0_PENDULUM, aDriver);
            } else {
                aDriver.z220_1_randomizeDigressionTrajectory();
                aDriver.z220_2_calculateDigressionTrajectory();
            }
        }

        public void onEnter(Driver aDriver) {
            aDriver.z220_0_initializeDigressionTrajectory();
            aDriver.z220_1_randomizeDigressionTrajectory();
            aDriver.z220_2_calculateDigressionTrajectory();
        }
    }
    /** ,   3  "" */

    private static class DriverState3 extends DriverState {

        public DriverState3() {
            super("State 3");
        }

        public void processEvent(int aEvent, Driver aDriver) {
            if (aEvent == Constants.EVENT_HIT_BY_BULLET) {
                changeState(STATE_2_DIGRESSION, aDriver);
            } else if (aDriver.x110_timeoutExpired()) {
                changeState(STATE_0_PENDULUM, aDriver);
            } else {
                aDriver.z200_2_calculatePendulumTrajectory();
            }
        }
        public void onEnter(Driver aDriver) {
            aDriver.z230_0_initializeStopTrajectory();
            aDriver.z200_2_calculatePendulumTrajectory();
        }
    }

}
