package newCynic;

import robocode.*;

import java.util.Enumeration;
import java.util.Hashtable;

/**
 *  "C ".
  */

public class TargetList {

    /**  . */
    public Target _closestTarget;
    /**
     *   "   
     *    .
     */
    private final double _targetsKDelta = 13;
    /**  ""  .*/
    private double targets_k;
    /**  */
    private Hashtable targetsTable = new Hashtable(1);
    /**   . */
    private long _lastScanCompletionTime = 0;
    /** - */
    private Cynical _cynical;

    public TargetList(Cynical aCynical) {
        _cynical = aCynical;
    }

    /**  . */
    public void beginRound() {
        Enumeration targets_list = targetsTable.elements();
        Target t;
        //  ""      .
        targets_k = 270 - _cynical.getOthers() * _targetsKDelta;
        if (targets_k <= 0) targets_k = 10;
        _closestTarget = null;

        while (targets_list.hasMoreElements()) {
            t = (Target) targets_list.nextElement();
            t.beginRound();
        }

        _lastScanCompletionTime = 0;
    }

    /**  . */
    public void beginTurn() {
        Enumeration targets_list = targetsTable.elements();
        Target t;
        while (targets_list.hasMoreElements()) {
            t = (Target) targets_list.nextElement();
            t.targetRefresh();
        }
        _closestTarget = getClosestTarget(100);
    }

    /**  . */
    public void update(ScannedRobotEvent e) {
        Target t;
        String robot_name = e.getName();

        t = (Target) targetsTable.get(robot_name);
        if (t != null) {
            //  .  .
            t.targetUpdate(e);
        } else {
            //      .
            t = new Target(_cynical, e);
            targetsTable.put(robot_name, t);
        }
    }

    /**  . */
    public void targetDestroyed(RobotDeathEvent e) {
        Target t;
        String robot_name = e.getName();
        t = (Target) targetsTable.get(robot_name);
        targets_k += _targetsKDelta;
        if (t != null)
            t.targetDestroyed();
    }

    /**   . */
    public void hit(BulletHitEvent e) {
        Target t;
        String robot_name = e.getName();
        t = (Target) targetsTable.get(robot_name);
        if (t != null)
            t.targetHit();
    }

    /**   . */
    public void collision(HitRobotEvent e) {
        Target t;
        String robot_name = e.getName();
        t = (Target) targetsTable.get(robot_name);
        if (t != null)
            t.collision();
    }

    /**   . */
    public void hitByBullet(HitByBulletEvent e) {
        Target t;
        String robot_name = e.getName();

        t = (Target) targetsTable.get(robot_name);
        if (t != null)
            t.hitByBullet();
    }

    /**   . */
    public Target getClosestTarget(double time_on_aiming) {
        Enumeration targets_list = targetsTable.elements();
        Target chosen_target = null, t = null;

        //  ,      ,
        //    ,  .
        while (targets_list.hasMoreElements()) {
            t = (Target) targets_list.nextElement();
            if (t.isTracked()) {
                if (Math.abs(Cynical.getAngleDiff(_cynical.getGunner().getCurHeading(), t._angle))
                        < time_on_aiming * Cynical.MAX_GUN_ROTATION_SPEED) {
                    //     .
                    if (chosen_target == null)
                        chosen_target = t;
                    else if (t._distance < chosen_target._distance)
                        chosen_target = t;
                }
            }
        }
        return chosen_target;
    }

    /**       . */
    public GeomVector
            getReactionVector() {
        Enumeration targets_list = targetsTable.elements();
        Target t;
        GeomVector reaction = new GeomVector();

        while (targets_list.hasMoreElements()) {
            t = (Target) targets_list.nextElement();
            if (t.isTracked()) {
                reaction.addRadialVector(Cynical.normalizeAngle(t._angle + Math.PI), targets_k / t._distance);
            }
        }

        return reaction;
    }

    /** ,     . */
    public boolean scanCompleted() {
        Enumeration targets_list = targetsTable.elements();
        Target t;
        int tracked_targets = 0;
        if (!targets_list.hasMoreElements()) return false;
        if (_cynical._currentTime < 20) return false;
        // ,     .
        while (targets_list.hasMoreElements()) {
            t = (Target) targets_list.nextElement();
            if (t.isTracked()) {
                //  .
                if (!t.wasUpdatedAfter(_lastScanCompletionTime)) {
                    //       .
                    //       .
                    return false;
                }
                tracked_targets++;
            }
        }
        //  , :
        //  -    ;
        //  -   ;
        //  -    (  ) .
        //     ,    .
        if (tracked_targets > 0) {
            _lastScanCompletionTime = _cynical._currentTime;
            return true;
        } else
            return false;
    }

    /**  */
    public void showStatistics() {
        Enumeration targets_list = targetsTable.elements();
        Target t;
        while (targets_list.hasMoreElements()) {
            t = (Target) targets_list.nextElement();
            t.logStatisitics();
        }
    }
}
