package newCynic;

import robocode.Event;
/**
 *       ""
 */

public abstract class TargetState {
    /**  ,    */
    private String _stateName;
    /** ,  .     . */
    protected TargetState(String aStateName) {
        _stateName = aStateName;
    }
    /** ,   . */
    public String getName() {
        return _stateName;
    }
    /** ,  .   
     *        .
     */
    public abstract void processEvent(int aEvent, Target aTarget, Event aRobodeEvent);

    /** ,       .
     *  
     *         .
     */
    public abstract void onEnter(Target aTarget);

    /**  0  "" */
    private final static TargetState STATE_0 = new TargetState0();
    /**  1  "" */
    private final static TargetState STATE_1 = new TargetState1();
    /**  2  "" */
    private final static TargetState STATE_2 = new TargetState2();
    /**  ,   
     *  (     ).
     */
    public static void reset(Target aTarget) {
        aTarget.setCurrentState(STATE_0);
    }
    /**  .*/
    public static boolean targetIsTracked(Target aTarget) {
        return STATE_2 == aTarget.getCurrentState();
    }

    /**
     *    --  .
     */
    private static void doStartLogging(int aEvent, Target aTarget) {
        if (Constants.OBJECTS_LOGGING)
            Logger.log("  '' (" + aTarget.getName() + "):");
        if (Constants.A5_BEGIN_LOGGING)
            Logger.logBegin(Constants.TARGET_AUTOMATE_NAME, aTarget.getCurrentState().getName(), aEvent);
    }

    /**
     *    --  .
     */
    private static void doEndLogging(int aEvent, Target aTarget) {
        if (Constants.A5_END_LOGGING)
            Logger.logEnd(Constants.TARGET_AUTOMATE_NAME, aTarget.getCurrentState().getName());
    }

    /**
     *   ,  .
     */
    private static void changeParentState(TargetState aNewState, Target aTarget) {
        if (Constants.A5_TRANS_LOGGING)
            Logger.logStateChange(Constants.TARGET_AUTOMATE_NAME, aNewState.getName(), aTarget.getCurrentState().getName());
        aTarget.setCurrentState(aNewState);
        aNewState.onEnter(aTarget);

    }

    /**
     *  ,    aEvent  aTarget.
     *  aRobocodeEvent     .
     *     .
     */
    public static void processIncomingEvent(int aEvent, Target aTarget, Event aRobodeEvent) {
        doStartLogging(aEvent, aTarget);
        aTarget.getCurrentState().processEvent(aEvent, aTarget, aRobodeEvent);
        doEndLogging(aEvent, aTarget);
    }

    /** ,   0  "" */
    private static class TargetState0 extends TargetState {
        public TargetState0() {
            super("State 0");
        }

        public void processEvent(int aEvent, Target aTarget, Event aRobodeEvent) {
            if (aEvent == Constants.EVENT_UPDATE) {
                aTarget.z1001_updateTargetData(aRobodeEvent);
                changeParentState(STATE_2, aTarget);
            } else if (aEvent == Constants.EVENT_HIT) {
                aTarget.z1010_updateTargetHitStatistics(aRobodeEvent);
                changeParentState(STATE_2, aTarget);
            } else if (aEvent == Constants.EVENT_COLLISION || aEvent == Constants.EVENT_WE_WERE_HIT) {
                changeParentState(STATE_2, aTarget);
            } else if (aEvent == Constants.EVENT_REFRESH && aTarget.x1000_targetDataIsOutOfDate()) {
                changeParentState(STATE_1, aTarget);
            }
        }

        public void onEnter(Target aTarget) {
            aTarget.z1000_resetTargetData();
        }
    }
    /** ,   1  "" */
    private static class TargetState1 extends TargetState {
        public TargetState1() {
            super("State 1");
        }

        public void processEvent(int aEvent, Target aTarget, Event aRobodeEvent) {
            if (aEvent == Constants.EVENT_UPDATE) {
                aTarget.z1001_updateTargetData(aRobodeEvent);
                changeParentState(STATE_2, aTarget);
            } else if (aEvent == Constants.EVENT_HIT) {
                aTarget.z1010_updateTargetHitStatistics(aRobodeEvent);
                changeParentState(STATE_2, aTarget);
            } else if (aEvent == Constants.EVENT_COLLISION || aEvent == Constants.EVENT_WE_WERE_HIT) {
                changeParentState(STATE_2, aTarget);
            } else if (aEvent == Constants.EVENT_START_ROUND) {
                changeParentState(STATE_0, aTarget);
            }
        }

        public void onEnter(Target aTarget) {
            aTarget.z1000_resetTargetData();
        }
    }

    /** ,   2  "" */
    private static class TargetState2 extends TargetState {

        public TargetState2() {
            super("State 2");
        }

        public void processEvent(int aEvent, Target aTarget, Event aRobodeEvent) {
            if (aEvent == Constants.EVENT_DESTROYED || aEvent == Constants.EVENT_REFRESH && aTarget.x1000_targetDataIsOutOfDate())
                changeParentState(STATE_1, aTarget);
            else if (aEvent == Constants.EVENT_START_ROUND)
                changeParentState(STATE_0, aTarget);
            else if (aEvent == Constants.EVENT_UPDATE) {
                aTarget.z1001_updateTargetData(aRobodeEvent);
            } else if (aEvent == Constants.EVENT_HIT) {
                aTarget.z1010_updateTargetHitStatistics(aRobodeEvent);
            }
        }

        public void onEnter(Target aTarget) {
        }
    }
}
